from __future__ import absolute_import


from katello.agent.pulp.libdnf import *


def test():
    from dnf.cli.main import main as cli
    cli(['install', 'ksh'])


def test_pattern():
    p = Pattern(name='jeff', version='1', arch='x68', release='2')
    print(p)


def test_packages(*pattern):
    names = pattern or ['bmake', 'mk-files']
    print('\n\nBegin')
    package = Package()
    log.info('\nInstalled [%s]: %s', names, package.install(names))
    names = [n.split('-', 1)[0] for n in names]
    log.info('\n\nUpdated [%s]: %s', names, package.update(names))
    log.info('\n\nUninstall [%s]: %s', names, package.uninstall(names))
    print('\nEnd')


def test_packages2(*pattern):
    print('\n\nBegin')
    package = Package()
    log.info('\nInstalled %s:\n%s', [str(p) for p in pattern], package.install(pattern))
    for p in pattern:
        p.version = ''
        p.release = ''
    log.info('\n\nUpdated %s:\n%s', [str(p) for p in pattern], package.update(pattern))
    log.info('\n\nUninstall %s: \n%s', [str(p) for p in pattern], package.uninstall(pattern))
    print('\nEnd')


def test_groups():
    names = ['Pulp Consumer']
    print('\nBegin')
    g = PackageGroup()
    log.info('Installed (group) %s', g.install(names))
    log.info('Uninstall (group) %s', g.uninstall(names))
    print('\nEnd')


def test_advisories():
    with LibDnf() as dnf:
        advisories = dnf.list_advisories()
        for ad in advisories:
            print('Ad: {}'.format(ad.id))
            for p in ad.packages:
                print('\t{}'.format(p.name))
        print('total: {}'.format(len(advisories)))


def test_applicable_advisories():
    with LibDnf() as dnf:
        advisories = dnf.applicable_advisories()
        for ad, packages in advisories:
            print('Ad: {}'.format(ad.id))
            for p in sorted(packages):
                print('\t{}'.format(p))
        print('total: {}'.format(len(advisories)))


def test_advisory_update():
    print('\nBegin')
    p = Package()
    log.info('Ad-Update: %s', p.update(advisories=['FEDORA-2017-de8a421dcd']))
    print('\nEnd')


if __name__ == '__main__':
    logging.basicConfig(level=logging.INFO)
    test_pattern()
    # test_packages()
    test_packages2(
        Pattern(name='walrus', version='0.71', release='1'),
        Pattern(name='zsh')
    )
    # test_applicable_advisories()
    # test_pattern()
    # test_advisories()
    # test_advisory_update()



#!/usr/bin/python
import sys
import logging, traceback

from katello.repos import upload_enabled_repos_report
from katello.utils import combined_profiles_enabled
from katello.enabled_report import EnabledReport
from zypp_plugin import Plugin

from katello.constants import ZYPPER_REPOSITORY_PATH

class EnabledReposUpload(Plugin):
    def __init__(self):
        Plugin.__init__(self)

    def PLUGINEND(self, headers, body):
        logging.info("Uploading Enabled Repositories Report")
        try:
            report = None
            if not combined_profiles_enabled():
                report = EnabledReport(ZYPPER_REPOSITORY_PATH)
            logging.info("Uploading Enabled Repositories Report ->  %s" % str(report))
            upload_enabled_repos_report(report)
        except:
            logging.error("Unable to upload Enabled Repositories Report - %s" % traceback.format_exc())
        self.ack()

if __name__ == '__main__':
    plugin = EnabledReposUpload()
    plugin.main()



import logging

import dnf
import libdnf
import hawkey


from gettext import gettext as _

from dnf.exceptions import CompsError


log = logging.getLogger('__file__')


class Pattern(object):
    """
    Package matching pattern.

    Attributes:
        name (str): Name.
        epoch (str): Epoch.
        version (str): Version.
        release (str): Release.
        arch (str) Arch..
    """

    __slots__ = (
        'name',
        'epoch',
        'version',
        'release',
        'arch',
    )

    FIELDS = (
        ('epoch', ('', ':')),
        ('name', ('', '')),
        ('version', ('-', '')),
        ('release', ('-', '')),
        ('arch', ('.', '')),
    )

    def __init__(self, name, epoch='', version='', release='', arch=''):
        """
        NEVRA

        Args:
            name (str): Name.
            epoch (str): Epoch.
            version (str): Version.
            release (str): Release.
            arch (str) Arch.
        """
        self.name = name
        self.epoch = epoch
        self.version = version
        self.release = release
        self.arch = arch

    def __eq__(self, other):
       return str(self) == str(other)

    def __str__(self):
        pattern = []
        for name, sep in self.FIELDS:
            value = getattr(self, name)
            if not value:
                continue
            if sep[0]:
                pattern.append(sep[0])
            pattern.append(value)
            if sep[1]:
                pattern.append(sep[1])
        return ''.join(pattern)


class API(object):
    """
    Abstract package management API.

    Attributes:
        commit (bool): Commit the transaction.
    """

    def __init__(self, commit=True):
        """
        Args:
            commit (bool): Commit the transaction.
                Use False for a "dry run".
        """
        self.commit = commit


class TransactionReport(object):
    """
    DNF transaction report.
    """

    def __init__(self, transaction):
        """
        Args:
            transaction: A DNF transaction.
        """

        ACTION_STATES = [
            dnf.transaction.PKG_INSTALL,
            dnf.transaction.PKG_REMOVE,
            dnf.transaction.PKG_UPGRADE
        ]
        FAILED_STATES = [
            libdnf.transaction.TransactionItemState_ERROR,
            libdnf.transaction.TransactionItemState_UNKNOWN
        ]

        self.resolved = []
        self.failed = []
        for item in transaction:
            po = item.pkg
            if po and item.state in FAILED_STATES:
                _list = self.failed
            elif po and item.action in ACTION_STATES:
                _list = self.resolved
            else:
                continue

            package = dict(
                qname=str(po),
                repoid=po.repoid,
                name=po.name,
                version=po.version,
                release=po.release,
                arch=po.arch,
                epoch=po.epoch)
            _list.append(package)

    def dict(self):
        """
        Dictionary representation.

        Returns:
            dict: self
        """
        return {
            'resolved': self.resolved,
            'failed': self.failed,
            'deps': [],
        }

    def log(self, heading):
        """
        Log the report.

        Args:
            heading (str): Log entry heading.
        """
        if not self:
            return
        names = [p['qname'] for p in self.resolved]
        _list = ''.join(['\n  {}'.format(n) for n in names])
        if _list:
            log.info(heading.format(_list))

    def __len__(self):
        return len(self.resolved)


class InstallTxReport(TransactionReport):
    """
    Install transaction report.
    """

    HEADING = _('\nInstalled:{}')

    def log(self, heading=HEADING):
        """
        Log the report.

        Args:
            heading (str): Log entry heading.
        """
        super(InstallTxReport, self).log(heading)


class UpdateTxReport(TransactionReport):
    """
    Update transaction report.
    """

    HEADING = _('\nUpdated:{}')

    def log(self, heading=HEADING):
        """
        Log the report.

        Args:
            heading (str): Log entry heading.
        """
        super(UpdateTxReport, self).log(heading)


class EraseTxReport(TransactionReport):
    """
    Erase transaction report.
    """

    HEADING = _('\nErased:{}')

    def log(self, heading=HEADING):
        """
        Log the report.

        Args:
            heading (str): Log entry heading.
        """
        super(EraseTxReport, self).log(heading)


class Package(API):
    """
    The package management API.
    """

    def install(self, patterns):
        """
        Install packages.

        Args:
            patterns (collections.Iterable): A list of Pattern.

        Returns:
            dict: A dictionary representation of a TransactionReport.
        """
        with LibDnf() as lib:
            lib.install(str(p) for p in patterns)
            if self.commit:
                lib.do_transaction()
            report = InstallTxReport(lib.transaction)
            if self.commit:
                report.log()
        return report.dict()

    def update(self, patterns=(), advisories=()):
        """
        Update packages.

        Args:
            patterns (collections.Iterable): A list of Pattern.
            advisories (collections.Iterable): A list of advisory IDs.

        Returns:
            dict: A dictionary representation of a TransactionReport.
        """
        with LibDnf() as lib:
            patterns = set(str(p) for p in patterns)
            if advisories:
                for ad, packages in lib.applicable_advisories(AdvisoryFilter(ids=advisories)):
                    for name, evr in packages:
                        patterns.add(name + '-' + evr)
                if patterns:
                   lib.upgrade(patterns)
            else:
                lib.upgrade(patterns)
            if self.commit:
                lib.do_transaction()
            report = UpdateTxReport(lib.transaction or ())
            if self.commit:
                report.log()
        return report.dict()

    def uninstall(self, patterns):
        """
        Uninstall (remove) packages.

        Args:
            patterns (collections.Iterable): A list of Pattern.

        Returns:
            dict: A dictionary representation of a TransactionReport.
        """
        with LibDnf() as lib:
            lib.remove(str(p) for p in patterns)
            if self.commit:
                lib.do_transaction()
            report = EraseTxReport(lib.transaction)
            if self.commit:
                report.log()
        return report.dict()


class PackageGroup(API):
    """
    Package group management API.
    """

    @staticmethod
    def _resolved(lib, names):
        """
        Resolve group names to IDs.

        Args:
            lib (LibDnf): An opened lib.
            names (collections.Iterable): A list of group names.

        Yields:
            Group IDs.

        Raises:
            CompsError: When name cannot be resolved.
        """
        for p in names:
            group = lib.comps.group_by_pattern(p)
            if not group:
                raise CompsError(_('Group "{g}" not found.').format(g=p))
            else:
                yield group.id

    def install(self, names):
        """
        Install package groups.

        Args:
            names (collections.Iterable): A list of group names.

        Returns:
            dict: A dictionary representation of a TransactionReport.
        """
        with LibDnf() as lib:
            lib.group_install(self._resolved(lib, names))
            if self.commit:
                lib.do_transaction()
            report = InstallTxReport(lib.transaction)
            if self.commit:
                report.log()
        return report.dict()

    def uninstall(self, names):
        """
        Uninstall package groups.

        Args:
            names (collections.Iterable): A list of group names.

        Returns:
            dict: A dictionary representation of a TransactionReport.
        """
        with LibDnf() as lib:
            lib.group_remove(self._resolved(lib, names))
            if self.commit:
                lib.do_transaction()
            report = EraseTxReport(lib.transaction)
            if self.commit:
                report.log()
        return report.dict()


class AdvisoryFilter(object):
    """
    Advisory filter.

    Filter by advisory IDs and/or types.
    """

    LABEL2TYPE = {
        _('BUGFIX'): hawkey.ADVISORY_BUGFIX,
        _('ENHANCEMENT'): hawkey.ADVISORY_ENHANCEMENT,
        _('SECURITY'): hawkey.ADVISORY_SECURITY,
        _('UNKNOWN'): hawkey.ADVISORY_UNKNOWN,
    }

    def __init__(self, ids=(), types=()):
        """
        Args:
            ids (collections.Iterable): List of advisory IDs.
            types (collections.Iterable): List of advisory types.  See: LABEL2TYPE.
        """
        self.ids = {s.upper() for s in ids}
        self.types = {self.LABEL2TYPE.get(s.upper(), hawkey.ADVISORY_UNKNOWN) for s in types}

    def match(self, advisory):
        """
        Match advisory.

        Args:
            advisory: An advisory.

        Returns:
            bool: True when matched.
        """
        if self.ids and (advisory.id.upper() not in self.ids):
            return False
        if self.types and (advisory.type not in self.types):
            return False
        return True


class LibDnf(dnf.Base):
    """
    DNF base.

    Notes:
        Requires dnf >= 2.7.5
    """

    # Logging cannot be reconfigured.
    # Plugins cannot be reloaded within the process.
    __lib_loaded = False

    def __init__(self):
        """
        Initialization.
        """
        super(LibDnf, self).__init__()
        self.conf.assumeyes = True
        self.conf.substitutions.update_from_etc("/")

    def open(self):
        """
        Open the lib.
        """
        self.read_all_repos()
        if not LibDnf.__lib_loaded:
            self._logging._setup_from_dnf_conf(self.conf)
            self.init_plugins()
            LibDnf.__lib_loaded = True
        self.fill_sack('auto', True)
        self._plugins.run_sack()
        self.read_comps()

    def install(self, patterns):
        """
        Install packages specified by the patterns.

        Args:
            patterns: List of (str) patterns.

        Notes:
            Need to call do_transaction() to commit the changes.
        """
        for p in patterns:
            super(LibDnf, self).install(p)
        self.resolve(allow_erasing=False)
        self._plugins.run_resolved()
        self._download()

    def upgrade(self, patterns=()):
        """
        Upgrade packages specified by the patterns.

        Args:
            patterns: List of (str) patterns.

        Notes:
            Need to call do_transaction() to commit the changes.
        """
        if patterns:
            for p in patterns:
                super(LibDnf, self).upgrade(p)
        else:
            self.upgrade_all()
        self.resolve(allow_erasing=False)
        self._plugins.run_resolved()
        self._download()

    def remove(self, patterns):
        """
        Remove packages specified by the patterns.

        Args:
            patterns: List of (str) patterns.

        Notes:
            Need to call do_transaction() to commit the changes.
        """
        for p in patterns:
            super(LibDnf, self).remove(p)
        self.resolve(allow_erasing=False)
        self._plugins.run_resolved()

    def group_install(self, grp_ids):
        """
        Install package groups.

        Args:
            grp_ids: List of (str) group IDs.

        Notes:
            Need to call do_transaction() to commit the changes.
        """
        for grp_id in grp_ids:
            super(LibDnf, self).group_install(grp_id, ('mandatory', 'default'))
        self.resolve(allow_erasing=False)
        self._plugins.run_resolved()
        self._download()

    def group_remove(self, grp_ids):
        """
        Remove package groups.

        Args:
            grp_ids: List of (str) group IDs.

        Notes:
            Need to call do_transaction() to commit the changes.
        """
        for grp_id in grp_ids:
            super(LibDnf, self).group_remove(grp_id)
        self.resolve(allow_erasing=False)
        self._plugins.run_resolved()

    def list_advisories(self, filter=AdvisoryFilter()):
        """
        Get a list of advisories matching the filter.

        Args:
            filter (AdvisoryFilter): An optional filter.

        Returns:
            list: The list of matching advisories.
        """
        advisories = []
        query = self.sack.query().filter(upgradable=True)
        for package in query.installed():
            for ad in package.get_advisories(hawkey.GT):
                if filter.match(ad):
                    advisories.append(ad)
        return advisories

    def applicable_advisories(self, filter=AdvisoryFilter()):
        """
        Get a list of applicable advisories matching the filter.

        Args:
            filter (AdvisoryFilter): An optional filter.

        Returns:
            list: The list of applicable advisories.
        """
        advisories = []
        query = self.sack.query()
        installed = {(p.name, p.arch): p for p in query.installed()}
        for ad in self.list_advisories(filter):
            packages = set()
            for ap in ad.packages:
                try:
                    ip = installed[(ap.name, ap.arch)]
                except KeyError:
                    continue
                if self.sack.evr_cmp(ip.evr, ap.evr) < 0:
                    packages.add((ap.name, ap.evr))
            if packages:
                advisories.append((ad, packages))
        return advisories

    def _download(self):
        """
        Download packages as needed.
        """
        self.download_packages(self.transaction.install_set)

    def __enter__(self):
        super(LibDnf, self).__enter__()
        self.open()
        return self



from __future__ import absolute_import
from os import path
import subprocess

# The path is defined in zypper, see https://github.com/openSUSE/libzypp/blob/master/zypp/target/TargetImpl.cc
REBOOT_NEEDED_FLAG = "/var/run/reboot-needed"


class ZypperTracerApp:
    def __init__(self, name, helper, type):
        self.name = name
        self.helper = helper
        self.type = type


def check_for_reboot_flag():
    apps = []
    if path.isfile(REBOOT_NEEDED_FLAG):
        app = ZypperTracerApp("kernel", "You will have to reboot your computer", "static")
        apps.append(app)
    return apps


def execute_zypper_ps():
    output = subprocess.check_output(["zypper", "ps", "-sss"], universal_newlines=True)
    return output.split("\n")


def collect_services_state():
    apps = []
    output = execute_zypper_ps()
    for service in output:
        if service:
            app = ZypperTracerApp(service, "systemctl restart " + service, "systemd")
            apps.append(app)
    return apps


def collect_apps(plugin=None):
    apps = collect_services_state()
    reboot = check_for_reboot_flag()
    return apps + reboot



#!/usr/bin/python
#
# Copyright 2017 ATIX AG
#
# This software is licensed to you under the GNU General Public
# License as published by the Free Software Foundation; either version
# 2 of the License (GPLv2) or (at your option) any later version.
# There is NO WARRANTY for this software, express or implied,
# including the implied warranties of MERCHANTABILITY,
# NON-INFRINGEMENT, or FITNESS FOR A PARTICULAR PURPOSE. You should
# have received a copy of GPLv2 along with this software; if not, see
# http://www.gnu.org/licenses/old-licenses/gpl-2.0.txt.
#

from os import readlink, getppid, environ
from os.path import basename
import sys
import logging

from katello.packages import upload_package_profile

from zypp_plugin import Plugin

class KatelloZyppPlugin(Plugin):

    def __init__(self):
        Plugin.__init__(self)
        self.num1 = self.num2 = None
        self.description = ""
        self.cleanup = "number"
        self.userdata = {}


    def parse_userdata(self, s):
        userdata = {}
        for kv in s.split(","):
            k, v = kv.split("=", 1)
            k = k.strip()
            if not k:
                raise ValueError
            userdata[k] = v.strip()
        return userdata


    def get_userdata(self, headers):
        try:
            return self.parse_userdata(headers['userdata'])
        except KeyError:
            pass
        except ValueError:
            logging.error("invalid userdata")
        return {}


    def PLUGINBEGIN(self, headers, body):
        self.description = "zypp(%s)" % basename(readlink("/proc/%d/exe" % getppid()))
        self.userdata = self.get_userdata(headers)
        self.ack()

    def PLUGINEND(self, headers, body):
        logging.info("Uploading Package Profile")
        try:
            upload_package_profile()
        except:
            logging.error("Unable to upload Package Profile")
        self.ack()


if __name__ == '__main__':
    if "DISABLE_KATELLO_ZYPP_PLUGIN" in environ:
        logging.info("$DISABLE_KATELLO_ZYPP_PLUGIN is set - disabling katello-zypp-plugin")

        # As the plugin is disabled, we are adding a dummy
        # Plugin so that zypper still works.
        plugin = Plugin()
    else:
        plugin = KatelloZyppPlugin()

    plugin.main()



import errno
import os
import os.path
import sys

from katello.constants import DISABLE_ENABLE_REPOS_VAR, ENABLED_REPOS_CACHE_FILE, ENABLED_REPOS_PLUGIN_CONF, PROFILE_CACHE_FILE
from katello.uep import get_manager, get_uep, lookup_consumer_id
from katello.utils import combined_profiles_enabled, plugin_enabled

from rhsm.connection import GoneException, RemoteServerException

try:
    import json
except ImportError:
    import simplejson as json


def error_message(msg):
    sys.stderr.write(msg + "\n")


def report_enabled_repos(consumer_id, report):
    """
    Report enabled repositories to the UEP.
    :param consumer_id: The consumer ID.
    :type consumer_id: str
    :param report: The report to send.
    :type report: dict
    """
    uep = get_uep()
    method = '/systems/%s/enabled_repos' % uep.sanitize(consumer_id)
    try:
        uep.conn.request_put(method, report)
        return True
    except (RemoteServerException, GoneException):
        error = sys.exc_info()[1]  # backward and forward compatible way to get the exception
        error_message(str(error))


def upload_enabled_repos_report(report, force=False):
    if not plugin_enabled(ENABLED_REPOS_PLUGIN_CONF, DISABLE_ENABLE_REPOS_VAR, force):
        return
    consumer_id = lookup_consumer_id()
    if consumer_id is None:
        error_message('Cannot upload enabled repos report, is this client registered?')
    elif report is None:
        get_manager().profilelib._do_update()
    else:
        content = report.content
        cache = EnabledRepoCache(consumer_id, content)
        if not cache.is_valid() and report_enabled_repos(consumer_id, content):
            cache.save()


class EnabledRepoCache:
    def __init__(self, consumer_id, content):
        self.consumer_id = consumer_id
        self.content = content

    @staticmethod
    def remove_cache():
        file_to_remove = ENABLED_REPOS_CACHE_FILE
        if combined_profiles_enabled():
            file_to_remove = PROFILE_CACHE_FILE
        try:
            os.remove(file_to_remove)
        except OSError:
            pass

    def is_valid(self):
        cache_file = None
        # wrapper try block w/ finally needed for python 2.4
        try:
            try:
                cache_file = open(ENABLED_REPOS_CACHE_FILE, 'r')
                try:
                    return self.data() == json.loads(cache_file.read())
                except ValueError:
                    return False
            except IOError:
                error = sys.exc_info()[1]  # backward and forward compatible way to get the exception
                if error.errno == errno.ENOENT:
                    return False
        finally:
            if cache_file is not None:
                cache_file.close()

    def data(self):
        return {self.consumer_id: self.content}

    def save(self):
        cache_dir = os.path.dirname(ENABLED_REPOS_CACHE_FILE)
        if not os.path.isdir(cache_dir):
            os.makedirs(cache_dir)

        cache_file = None
        try:
            cache_file = open(ENABLED_REPOS_CACHE_FILE, 'w')
            cache_file.write(json.dumps(self.data()))
        finally:
            if cache_file is not None:
                cache_file.close()



import os
from yum.plugins import TYPE_CORE, TYPE_INTERACTIVE

from katello.repos import upload_enabled_repos_report

from logging import Logger

from katello.utils import combined_profiles_enabled, is_root_user
from katello.enabled_report import EnabledReport
from katello.constants import REPOSITORY_PATH

requires_api_version = '2.3'
plugin_type = (TYPE_CORE, TYPE_INTERACTIVE)

def close_hook(conduit):
    if not is_root_user():
        return conduit.error(2, "Skipping Enabled Repositories Upload because you are not logged in as root")
    if not conduit.confBool("main", "supress_debug"):
        conduit.info(2, "Uploading Enabled Repositories Report")
    try:
        report = None
        if not combined_profiles_enabled():
            report = EnabledReport(REPOSITORY_PATH)
        upload_enabled_repos_report(report)
    except:
        if not conduit.confBool("main", "supress_errors"):
            conduit.error(2, "Unable to upload Enabled Repositories Report")



#
# Copyright 2013 Red Hat, Inc.
#
# This software is licensed to you under the GNU General Public
# License as published by the Free Software Foundation; either version
# 2 of the License (GPLv2) or (at your option) any later version.
# There is NO WARRANTY for this software, express or implied,
# including the implied warranties of MERCHANTABILITY,
# NON-INFRINGEMENT, or FITNESS FOR A PARTICULAR PURPOSE. You should
# have received a copy of GPLv2 along with this software; if not, see
# http://www.gnu.org/licenses/old-licenses/gpl-2.0.txt.
#

"""
The katello virtual agent.
Provides content management APIs for pulp within the RHSM environment.
"""
import os
import sys

sys.path.append('/usr/share/rhsm')

from time import sleep
from logging import getLogger
from subprocess import Popen

from six.moves import http_client as http

from gofer.decorators import load, unload, remote, action, FORK
from gofer.agent.plugin import Plugin
from gofer.pmon import PathMonitor
from gofer.config import Config

try:
    from subscription_manager.identity import ConsumerIdentity
except ImportError:
    from subscription_manager.certlib import ConsumerIdentity

from rhsm.connection import UEPConnection, RemoteServerException, GoneException

from katello.agent.pulp import Dispatcher


# This plugin
plugin = Plugin.find(__name__)

# Path monitoring
path_monitor = None

# Track registration status
registered = False


log = getLogger(__name__)


RHSM_CONFIG_PATH = '/etc/rhsm/rhsm.conf'


@load
def plugin_loaded():
    """
    Initialize the plugin.
    Called (once) immediately after the plugin is loaded.
     - setup path monitoring.
     - validate registration.  If registered:
       - setup plugin configuration.
    """
    global path_monitor
    path = ConsumerIdentity.certpath()
    path_monitor = PathMonitor()
    path_monitor.add(path, certificate_changed)
    path_monitor.start()
    while True:
        try:
            validate_registration()
            if registered:
                update_settings()
            # DONE
            break
        except Exception as e:
            log.warn(str(e))
            sleep(60)


@unload
def plugin_unloaded():
    """
    The plugin has been uploaded.
    """
    path_monitor.abort()


def bundle(certificate):
    """
    Bundle the key and cert and write to a file.
    :param certificate: A consumer identity certificate.
    :type certificate: ConsumerIdentity
    :return: The path to written bundle.
    :rtype: str
    """
    path = os.path.join(certificate.PATH, 'bundle.pem')
    fp = open(path, 'w')
    try:
        fp.write(certificate.key)
        fp.write(certificate.cert)
        return path
    finally:
        fp.close()


def certificate_changed(path):
    """
    A certificate change has been detected.
    On registration: setup the plugin; attach to the message broker.
    On un-registration: detach from the message broker.
    :param path: The path to the file that changed.
    :type path: str
    """
    log.info('changed: %s', path)
    while True:
        try:
            validate_registration()
            if registered:
                update_settings()
                plugin.attach()
            else:
                plugin.detach()
            # DONE
            break
        except Exception as e:
            log.warn(str(e))
            sleep(60)


def update_settings():
    """
    Setup the plugin based on the RHSM configuration.
    """
    rhsm_conf = Config(RHSM_CONFIG_PATH)
    certificate = ConsumerIdentity.read()
    if 'ca_cert_dir' in rhsm_conf['rhsm']:
        ca_cert_dir = rhsm_conf['rhsm']['ca_cert_dir']
    else:
        #handle old subscription-manager configurations
        ca_cert_dir = rhsm_conf['server']['ca_cert_dir']

    # the 'katello-default-ca.pem' is the ca used for generating the CA certs.
    # the 'candlepin-local.pem' is there for compatibility reasons (the old path where the
    # legacy installer was putting this file. If none of them is present, there is still
    # a chance the rhsm_conf['rhsm']['repo_ca_cert'] is serving as the CA for issuing
    # the client certs
    ca_candidates = [os.path.join(ca_cert_dir, 'katello-default-ca.pem'),
                     os.path.join(ca_cert_dir, 'candlepin-local.pem'),
                     rhsm_conf['rhsm']['repo_ca_cert'] % {'ca_cert_dir': ca_cert_dir}]
    existing_ca_certs = [cert for cert in ca_candidates if os.path.exists(cert)]
    if not existing_ca_certs:
       log.warn('None of the ca cert files %s found for the qpid connection' % ca_candidates)

       raise
    else:
       log.info('Using %s as the ca cert for qpid connection' % existing_ca_certs[0])

    plugin.cfg.messaging.cacert = existing_ca_certs[0]
    plugin.cfg.messaging.url = 'proton+amqps://%s:5647' % rhsm_conf['server']['hostname']
    plugin.cfg.model.queue = 'pulp.agent.%s' % certificate.getConsumerId()
    bundle(certificate)


def validate_registration():
    """
    Validate consumer registration by making a REST call
    to the server.  Updates the global 'registered' variable.
    """
    global registered
    registered = False

    if ConsumerIdentity.existsAndValid():
        consumer = ConsumerIdentity.read()
        consumer_id = consumer.getConsumerId()
    else:
        return

    try:
        uep = UEP()
        consumer = uep.getConsumer(consumer_id)
        registered = (consumer is not None)
    except GoneException:
        registered = False
    except RemoteServerException as e:
        if e.code not in (http.NOT_FOUND, http.GONE):
            log.warn(str(e))
            raise
    except Exception as e:
        log.exception(str(e))
        raise


class AgentRestart(object):
    """
    Restart the daemon after RPM upgrade.
    The %post in the RPM will write the RESTART_FILE.  The recurring
    'apply' action notices file and restarts the goferd service only when
    this plugin is not longer busy.
    """

    COMMAND = 'service goferd restart'
    RESTART_FILE = '/tmp/katello-agent-restart'

    @staticmethod
    def is_busy():
        """
        Determine if this plugin is busy by counting the pending
        requests in the persistent queue.

        :return: True if busy.
        :rtype: bool
        """
        pending = plugin.scheduler.pending
        path = os.path.join(pending.PENDING, pending.stream)
        count = len(os.listdir(path))
        return count > 0

    @staticmethod
    def restart():
        """
        Restart the goferd service.
          1. Delete the RESTART_FILE.
          2. Restart

        :return: The restart command exit value.
        :rtype: int
        """
        os.unlink(AgentRestart.RESTART_FILE)
        p = Popen(AgentRestart.COMMAND, shell=True)
        return p.wait()

    @action(minutes=3)
    def apply(self):
        """
        Detect the RESTART_FILE and restart the goferd service
        only when this plugin is not longer busy.  This ensures that
        an RPM update has completed.
        """
        if not os.path.exists(AgentRestart.RESTART_FILE):
            return
        if self.is_busy():
            return
        log.info('Restarting goferd.')
        exit_val = self.restart()
        # only reached when restart failed.
        log.error('Restart failed, exit=%d', exit_val)


class UEP(UEPConnection):
    """
    Represents the UEP.
    """

    def __init__(self):
        key = ConsumerIdentity.keypath()
        cert = ConsumerIdentity.certpath()
        UEPConnection.__init__(self, key_file=key, cert_file=cert)


# --- API --------------------------------------------------------------------
class Consumer(object):
    """
    When a consumer is unregistered, Katello notifies the goferd.
    """

    @remote
    def unregister(self):
        log.info('Consumer has been unregistered. '
                 'Katello agent will no longer function until '
                 'this system is reregistered.')


class Content(object):
    """
    Pulp Content Management.
    """

    @remote(model=FORK)
    def install(self, units, options):
        """
        Install the specified content units using the specified options.
        Delegated to content handlers.
        :param units: A list of content units to be installed.
        :type units: list of:
            { type_id:<str>, unit_key:<dict> }
        :param options: Install options; based on unit type.
        :type options: dict
        :return: A dispatch report.
        :rtype: DispatchReport
        """
        dispatcher = Dispatcher()
        report = dispatcher.install(units, options)
        return report.dict()

    @remote(model=FORK)
    def update(self, units, options):
        """
        Update the specified content units using the specified options.
        Delegated to content handlers.
        :param units: A list of content units to be updated.
        :type units: list of:
            { type_id:<str>, unit_key:<dict> }
        :param options: Update options; based on unit type.
        :type options: dict
        :return: A dispatch report.
        :rtype: DispatchReport
        """
        dispatcher = Dispatcher()
        report = dispatcher.update(units, options)
        return report.dict()

    @remote(model=FORK)
    def uninstall(self, units, options):
        """
        Uninstall the specified content units using the specified options.
        Delegated to content handlers.
        :param units: A list of content units to be uninstalled.
        :type units: list of:
            { type_id:<str>, unit_key:<dict> }
        :param options: Uninstall options; based on unit type.
        :type options: dict
        :return: A dispatch report.
        :rtype: DispatchReport
        """
        dispatcher = Dispatcher()
        report = dispatcher.uninstall( units, options)
        return report.dict()



#
# Copyright 2014 Red Hat, Inc.
#
# This software is licensed to you under the GNU General Public
# License as published by the Free Software Foundation; either version
# 2 of the License (GPLv2) or (at your option) any later version.
# There is NO WARRANTY for this software, express or implied,
# including the implied warranties of MERCHANTABILITY,
# NON-INFRINGEMENT, or FITNESS FOR A PARTICULAR PURPOSE. You should
# have received a copy of GPLv2 along with this software; if not, see
# http://www.gnu.org/licenses/old-licenses/gpl-2.0.txt.
#

from katello.packages import upload_package_profile

from yum.plugins import TYPE_CORE, TYPE_INTERACTIVE

requires_api_version = '2.3'
plugin_type = (TYPE_CORE, TYPE_INTERACTIVE)


def posttrans_hook(conduit):
    if not conduit.confBool("main", "supress_debug"):
        conduit.info(2, "Uploading Package Profile")
    try:
        upload_package_profile()
    except:
        if not conduit.confBool("main", "supress_errors"):
            conduit.error(2, "Unable to upload Package Profile")



import optparse

from katello.constants import (DISABLE_ENABLE_REPOS_VAR, DISABLE_PACKAGE_PROFILE_VAR,
        ENABLED_REPOS_PLUGIN_CONF, PACKAGE_PROFILE_PLUGIN_CONF, REPOSITORY_PATH, YUM, ZYPPER,
        ZYPPER_REPOSITORY_PATH)
from katello.enabled_report import EnabledReport
from katello.packages import purge_package_cache, upload_package_profile
from katello.repos import EnabledRepoCache, upload_enabled_repos_report
from katello.utils import combined_profiles_enabled


def enabled_repos_upload():
    description = 'This report can be disabled by either disabling the package management plugin in ' +\
              ENABLED_REPOS_PLUGIN_CONF + ' or via the environment variable ' + DISABLE_ENABLE_REPOS_VAR
    parser = optparse.OptionParser(description=description)
    parser.add_option('-f', '--force', action='store_true',
            help="Force enabled repository upload even if it does not seem out of date, or is otherwise disabled..")
    (options, args) = parser.parse_args()
    if options.force:
        EnabledRepoCache.remove_cache()

    report = None
    if not combined_profiles_enabled():
        if YUM:
            repo_path = REPOSITORY_PATH
        elif ZYPPER:
            repo_path = ZYPPER_REPOSITORY_PATH
        else:
            raise IOError('Neither yum nor zypper can be used')
        report = EnabledReport(repo_path)

    upload_enabled_repos_report(report, options.force)


def package_upload():
    description = 'This report can be disabled by either disabling the package management plugin in ' + \
              PACKAGE_PROFILE_PLUGIN_CONF + ' or via the environment variable ' + DISABLE_PACKAGE_PROFILE_VAR
    parser = optparse.OptionParser(description=description)
    parser.add_option('-f', '--force', action='store_true',
            help="Force package upload even if it does not seem out of date.")

    (options, args) = parser.parse_args()
    if options.force:
        purge_package_cache()
    upload_package_profile(options.force)


def tracer_upload():
    try:
        from katello import tracer
    except ImportError:
        raise SystemExit('Tracer is not supported on your platform')
    else:
        tracer.upload_tracer_profile(tracer.query_affected_apps, None)



import time

from katello.tracer import Query, upload_tracer_profile

from yum.plugins import TYPE_CORE, TYPE_INTERACTIVE

requires_api_version = '2.3'
plugin_type = (TYPE_CORE, TYPE_INTERACTIVE)


def query_apps(conduit):
    """Returns all apps that need restarting """
    query = Query()

    # When running via yum we need to pass tracer a list of packages and
    # their last modified time so it has no need to access the rpmdb (which
    # would fail as yum has a lock on it)
    packages = []

    # Packages in the current transation
    pkgs = conduit.getTsInfo().getMembers()
    for pkg in pkgs:
        pkg.modified = time.time()
        packages.append(pkg)
    rpmdb = conduit.getRpmDB()  # All other packages
    for pkg in rpmdb:
        pkg.modified = pkg.installtime
        packages.append(pkg)
    return query.from_packages(packages).now().affected_applications().get()


def posttrans_hook(conduit):
    if not conduit.confBool("main", "supress_debug"):
        conduit.info(2, "Uploading Tracer Profile")
    try:
        upload_tracer_profile(query_apps, conduit)
    except:
        if not conduit.confBool("main", "supress_errors"):
            conduit.error(2, "Unable to upload Tracer Profile")
